{ TAMDesktop 1.0
  copyright  AM Software 1998 by Alexander Meeder

  You can use this component to draw on the Windows 95/98/NT desktop (behind all applications).
  I developed it with Delphi 3 on a Window 98 machine so I dont know if it will work with
  other Windows versions.
  To draw on the Desktop use one of the following methods:

  1. Add code to the OnPaint-event and call the Paint-method of the component
  2. Add code to the OnPaint-event and set the Enabled-property to true and the interval
     to call the Paint-method automatically

  Use the OnPaint-event as follows:

  procedure AMDesktop1.OnPaint(Sender: TObject);
  begin
    with AMDesktop1.Bitmap.Canvas do
    begin
      // Use any standard Delphi canvas drawing command like Draw, CopyRect, Rectangle etc.
      // It even draws transparent pictures transparent on the desktop, just set the
      // transparent property of the Image-component to true
      Draw(100, 100, Image1.Picture.Graphic);
    end;
  end;

  You can stil select/doubleclick the icons on the desktop but there are some problems:
  * When you move icons on the desktop you still see the icon on the old place until you
    set the enabled-property to false and true again.
  * When there are other applications active (visible, not minimized) then those applications
    are stil visible when you minimize or close them until you set the enabled-property to false
    true again

  If you have some questions about, or suggestions for this component
  you can mail me at: ameeder@dds.nl
}

unit AMDesktop;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ThdTimer;

type
  TAMDesktop = class(TComponent)
  private
    FBitmap: TBitmap;
    FChanging: TNotifyEvent;
    FChanged: TNotifyEvent;
    FDesktopBitmap: TBitmap;
    FDesktopCanvas: TCanvas;
    FDesktopHandle: Hwnd;
    FPaint: TNotifyEvent;
    FTimer: TThreadedTimer;
    FTimerTimer: TNotifyEvent;
    function GetEnabled: boolean;
    function GetInterval: integer;
    function GetPriority: TThreadPriority;
    procedure SetEnabled(Value: boolean);
    procedure SetInterval(Value: integer);
    procedure SetPriority(Value: TThreadPriority);
  protected
    procedure Loaded; override;
    procedure Timer(Sender: TObject);
    procedure CaptureDesktop;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Changing(Sender: TObject);
    procedure Changed(Sender: TObject);
    procedure Paint;
    property Bitmap: TBitmap read FBitmap;
  published
    property Enabled: boolean read GetEnabled write SetEnabled default false;
    property Interval: integer read GetInterval write SetInterval default 250;
    property Priority: TThreadPriority read GetPriority write SetPriority default tpNormal;
    property OnChanging: TNotifyEvent read FChanging write FChanging;
    property OnChanged: TNotifyEvent read FChanged write FChanged;
    property OnPaint: TNotifyEvent read FPaint write FPaint;
    property OnTimer: TNotifyEvent read FTimerTimer write FTimerTimer;
  end;

procedure Register;

implementation

function TAMDesktop.GetEnabled: boolean;
begin
  Result := FTimer.Enabled;
end;

function TAMDesktop.GetInterval: integer;
begin
  Result := FTimer.Interval;
end;

function TAMDesktop.GetPriority: TThreadPriority;
begin
  Result := FTimer.Priority;
end;

procedure TAMDesktop.SetEnabled(Value: boolean);
var
  SRect: TRect;
begin
  if not Value then
  begin
    try
      FDesktopCanvas.Handle := GetWindowDC(FDesktopHandle);
      SRect := Rect(0, 0, Screen.Width, Screen.Height);
      FDesktopCanvas.CopyRect(SRect, FDesktopBitmap.Canvas, SRect);
    finally
      ReleaseDC(0, FDesktopCanvas.Handle);
    end;
  end
  else
    CaptureDesktop;
    
  FTimer.Enabled := Value;
end;

procedure TAMDesktop.SetInterval(Value: integer);
begin
  FTimer.Interval := Value;
end;

procedure TAMDesktop.SetPriority(Value: TThreadPriority);
begin
  FTimer.Priority := Value;
end;

procedure TAMDesktop.Paint;
var
  SRect: TRect;
begin
  if Assigned(FPaint) then
  begin
    FDesktopCanvas.Handle := GetWindowDC(FDesktopHandle);
    try
      SRect := Rect(0, 0, Screen.Width, Screen.Height);
      FBitmap.Canvas.CopyRect(SRect, FDesktopBitmap.Canvas, SRect);
      FPaint(Self);
      FDesktopCanvas.CopyRect(SRect, FBitmap.Canvas, SRect);
    finally
      ReleaseDC(0, FDesktopCanvas.Handle);
    end;
  end;
end;

procedure TAMDesktop.CaptureDesktop;
var
  SRect: TRect;
begin
  if not (csDesigning in ComponentState) then
  begin
    try
      TForm(Owner).Hide;    
      FDesktopCanvas.Handle := GetWindowDC(FDesktopHandle);
      SendMessage(FDesktopHandle, WM_PAINT, FDesktopCanvas.Handle, 0);
      SRect := Rect(0, 0, Screen.Width, Screen.Height);
      FDesktopBitmap.Canvas.CopyRect(SRect, FDesktopCanvas, SRect);
    finally
      TForm(Owner).Show;
      ReleaseDC(0, FDesktopCanvas.Handle);
    end;
  end;
end;

procedure TAMDesktop.Timer(Sender: TObject);
begin
  if not (csDesigning in ComponentState) then
  begin
    Paint;
    if Assigned(FTimerTimer) then FTimerTimer(Self);
  end;
end;

procedure TAMDesktop.Changing(Sender: TObject);
begin
  if Assigned(FChanging) then FChanging(Self);
end;

procedure TAMDesktop.Changed;
begin
  if Assigned(FChanged) then FChanged(Self);
end;

procedure TAMDesktop.Loaded;
begin
  if not(csDesigning in ComponentState) then
  begin
   FDesktopHandle := FindWindow('Progman', nil);
   CaptureDesktop;
  end;
end;

constructor TAMDesktop.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDesktopHandle := 0;
  FBitmap := TBitmap.Create;
  FBitmap.Width := Screen.Width;
  FBitmap.Height := Screen.Height;
  FDesktopBitmap := TBitmap.Create;
  FDesktopBitmap.Width := Screen.Width;
  FDesktopBitmap.Height := Screen.Height;
  FDesktopCanvas := TCanvas.Create;
  FDesktopCanvas.OnChanging := Changing;
  FDesktopCanvas.OnChange := Changed;
  FTimer := TThreadedTimer.Create(nil);
  FTimer.Enabled := False;
  FTimer.Interval := 250;
  FTimer.Priority := tpNormal;
  FTimer.OnTimer := Timer;
end;

destructor TAMDesktop.Destroy;
begin
  FTimer.Enabled := False;
  FTimer.OnTimer := nil;
  FTimer.Free;
  FBitmap.Free;
  FDesktopBitmap.Free;
  FDesktopCanvas.Handle := GetWindowDC(FDesktopHandle);
  try
    SendMessage(FDesktopHandle, WM_ERASEBKGND, FDesktopCanvas.Handle, 0);
    SendMessage(FDesktopHandle, WM_PAINT, FDesktopCanvas.Handle, 0);
  finally
    ReleaseDC(0, FDesktopCanvas.Handle);
    FDesktopCanvas.Free;
  end;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('AM Software', [TAMDesktop]);
end;

end.
